extends CanvasLayer

@export var window : Window
@export var list : ItemList
@export var categoriesButton : OptionButton

func _ready() -> void:
	window.visible = false

func _input(event: InputEvent) -> void:
	if get_tree().current_scene.scene_file_path == "res://Scenes/Menu.tscn":
		return
		
	if event.is_pressed() && event is InputEventKey:
		if event.keycode == KEY_INSERT:
			toggleVisibility()

var dirty = true
func toggleVisibility() -> void:
	window.visible = !window.visible
	if dirty && window.visible:
		updateList()
		dirty = false
	
func iconZoomChanged(zoom: float):
	list.fixed_icon_size = Vector2i.ONE * zoom

var nameFilter : String = ""
func nameFilterChanged(f : String):
	nameFilter = f
	updateList()

func catFilterChanged(_i : int):
	updateList()
	
func updateList() -> void:
	list.clear()

	var map = {}
	var preloaderClass : GDScript = load("res://Scripts/Preloader.gd")
	while preloaderClass:
		map.merge(preloaderClass.get_script_constant_map())
		preloaderClass = preloaderClass.get_base_script()

	var categories = []
	for p in map:
		if p.ends_with("_Rig"):
			continue

		var v = map.get(p)
		if v is PackedScene:
			var pickup = v.instantiate()
			var slot : SlotData = pickup.slotData
			var item : ItemData = pickup.slotData.itemData 

			if nameFilter && !item.name.to_lower().contains(nameFilter.to_lower()):
				continue

			var catFilter = categoriesButton.get_selected_metadata()
			if catFilter && !item.type == catFilter:
				continue

			var id = list.add_item(item.name, item.icon)
			list.set_item_metadata(id, slot)

			if dirty && item.type && !(item.type in categories):
				categories.append(item.type)
	
	if dirty:
		categoriesButton.clear()
		categoriesButton.add_item("All")
		categoriesButton.set_item_metadata(categoriesButton.item_count-1, null)

		for cat in categories:
			categoriesButton.add_item(cat)
			categoriesButton.set_item_metadata(categoriesButton.item_count-1, cat)

func windowInput(input : InputEvent) -> void:
	if !window.visible:
		return
	if !window.is_input_handled() && input.is_action_pressed("ui_cancel"):
		toggleVisibility()
		window.set_input_as_handled()


func spawnSelected() -> void:
	var items = list.get_selected_items()
	if items.size() == 0:
		return
	
	var id = items[0]
	var slotdata = list.get_item_metadata(id)
	var iface : Interface = $/root/Map/Core/UI/UI_Interface

	var slot = load("res://UI/Elements/Slot.tscn").instantiate()
	slot.slotData = slotdata.duplicate()
	slot.type = Slot.SlotType.Equipment
	add_child(slot)

	iface.actionSlot = slot
	iface.ItemActionPlace.call_deferred()
	slot.queue_free.call_deferred()
	(func(): iface.actionSlot = null).call_deferred() 
	toggleVisibility()